@@ -47,7 +47,8 @@
  */
 
 static inline const char*
-button_state_to_str(enum button_state state) {
+button_state_to_str(enum button_state state)
+{
 	switch(state) {
 	CASE_RETURN_STRING(BUTTON_STATE_NONE);
 	CASE_RETURN_STRING(BUTTON_STATE_AREA);
@@ -61,7 +62,8 @@ button_state_to_str(enum button_state state) {
 }
 
 static inline const char*
-button_event_to_str(enum button_event event) {
+button_event_to_str(enum button_event event)
+{
 	switch(event) {
 	CASE_RETURN_STRING(BUTTON_EVENT_IN_BOTTOM_R);
 	CASE_RETURN_STRING(BUTTON_EVENT_IN_BOTTOM_M);
@@ -173,25 +175,25 @@ tp_button_set_state(struct tp_dispatch *tp,
 
 	switch (t->button.state) {
 	case BUTTON_STATE_NONE:
-		t->button.curr = 0;
+		t->button.current = 0;
 		break;
 	case BUTTON_STATE_AREA:
-		t->button.curr = BUTTON_EVENT_IN_AREA;
+		t->button.current = BUTTON_EVENT_IN_AREA;
 		break;
 	case BUTTON_STATE_BOTTOM:
-		t->button.curr = event;
+		t->button.current = event;
 		break;
 	case BUTTON_STATE_TOP:
 		break;
 	case BUTTON_STATE_TOP_NEW:
-		t->button.curr = event;
+		t->button.current = event;
 		tp_button_set_enter_timer(tp, t);
 		break;
 	case BUTTON_STATE_TOP_TO_IGNORE:
 		tp_button_set_leave_timer(tp, t);
 		break;
 	case BUTTON_STATE_IGNORE:
-		t->button.curr = 0;
+		t->button.current = 0;
 		break;
 	}
 }
@@ -258,7 +260,7 @@ tp_button_bottom_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_BOTTOM_R:
 	case BUTTON_EVENT_IN_BOTTOM_M:
 	case BUTTON_EVENT_IN_BOTTOM_L:
-		if (event != t->button.curr)
+		if (event != t->button.current)
 			tp_button_set_state(tp,
 					    t,
 					    BUTTON_STATE_BOTTOM,
@@ -294,7 +296,7 @@ tp_button_top_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_TOP_R:
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
-		if (event != t->button.curr)
+		if (event != t->button.current)
 			tp_button_set_state(tp,
 					    t,
 					    BUTTON_STATE_TOP_NEW,
@@ -327,7 +329,7 @@ tp_button_top_new_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_TOP_R:
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
-		if (event != t->button.curr)
+		if (event != t->button.current)
 			tp_button_set_state(tp,
 					    t,
 					    BUTTON_STATE_TOP_NEW,
@@ -359,7 +361,7 @@ tp_button_top_to_ignore_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_TOP_R:
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
-		if (event == t->button.curr)
+		if (event == t->button.current)
 			tp_button_set_state(tp,
 					    t,
 					    BUTTON_STATE_TOP,
@@ -405,7 +407,7 @@ tp_button_ignore_handle_event(struct tp_dispatch *tp,
 		tp_button_set_state(tp, t, BUTTON_STATE_NONE, event);
 		break;
 	case BUTTON_EVENT_PRESS:
-		t->button.curr = BUTTON_EVENT_IN_AREA;
+		t->button.current = BUTTON_EVENT_IN_AREA;
 		break;
 	case BUTTON_EVENT_RELEASE:
 		break;
@@ -455,6 +457,35 @@ tp_button_handle_event(struct tp_dispatch *tp,
 				button_state_to_str(t->button.state));
 }
 
+static inline void
+tp_button_check_for_movement(struct tp_dispatch *tp, struct tp_touch *t)
+{
+	struct device_coords delta;
+	struct phys_coords mm;
+	double vector_length;
+
+	switch (t->button.state) {
+	case BUTTON_STATE_NONE:
+	case BUTTON_STATE_AREA:
+	case BUTTON_STATE_TOP:
+	case BUTTON_STATE_TOP_NEW:
+	case BUTTON_STATE_TOP_TO_IGNORE:
+	case BUTTON_STATE_IGNORE:
+		/* No point calculating if we're not going to use it */
+		return;
+	case BUTTON_STATE_BOTTOM:
+		break;
+	}
+
+	delta.x = t->point.x - t->button.initial.x;
+	delta.y = t->point.y - t->button.initial.y;
+	mm = evdev_device_unit_delta_to_mm(tp->device, &delta);
+	vector_length = hypot(mm.x, mm.y);
+
+	if (vector_length > 5.0 /* mm */)
+		t->button.has_moved = true;
+}
+
 void
 tp_button_handle_state(struct tp_dispatch *tp, uint64_t time)
 {
@@ -464,25 +495,37 @@ tp_button_handle_state(struct tp_dispatch *tp, uint64_t time)
 		if (t->state == TOUCH_NONE || t->state == TOUCH_HOVERING)
 			continue;
 
+		if (t->state == TOUCH_BEGIN) {
+			t->button.initial = t->point;
+			t->button.has_moved = false;
+		}
+
 		if (t->state == TOUCH_END) {
 			tp_button_handle_event(tp, t, BUTTON_EVENT_UP, time);
 		} else if (t->dirty) {
 			enum button_event event;
 
-			if (is_inside_bottom_right_area(tp, t))
-				event = BUTTON_EVENT_IN_BOTTOM_R;
-			else if (is_inside_bottom_middle_area(tp, t))
-				event = BUTTON_EVENT_IN_BOTTOM_M;
-			else if (is_inside_bottom_left_area(tp, t))
-				event = BUTTON_EVENT_IN_BOTTOM_L;
-			else if (is_inside_top_right_area(tp, t))
-				event = BUTTON_EVENT_IN_TOP_R;
-			else if (is_inside_top_middle_area(tp, t))
-				event = BUTTON_EVENT_IN_TOP_M;
-			else if (is_inside_top_left_area(tp, t))
-				event = BUTTON_EVENT_IN_TOP_L;
-			else
+			if (is_inside_bottom_button_area(tp, t)) {
+				if (is_inside_bottom_right_area(tp, t))
+					event = BUTTON_EVENT_IN_BOTTOM_R;
+				else if (is_inside_bottom_middle_area(tp, t))
+					event = BUTTON_EVENT_IN_BOTTOM_M;
+				else
+					event = BUTTON_EVENT_IN_BOTTOM_L;
+
+				/* In the bottom area we check for movement
+				 * within the area. Top area - meh */
+				tp_button_check_for_movement(tp, t);
+			} else if (is_inside_top_button_area(tp, t)) {
+				if (is_inside_top_right_area(tp, t))
+					event = BUTTON_EVENT_IN_TOP_R;
+				else if (is_inside_top_middle_area(tp, t))
+					event = BUTTON_EVENT_IN_TOP_M;
+				else
+					event = BUTTON_EVENT_IN_TOP_L;
+			} else {
 				event = BUTTON_EVENT_IN_AREA;
+			}
 
 			tp_button_handle_event(tp, t, event, time);
 		}
@@ -1119,7 +1162,7 @@ tp_post_clickpadbutton_buttons(struct tp_dispatch *tp, uint64_t time)
 		uint32_t area = 0;
 
 		tp_for_each_touch(tp, t) {
-			switch (t->button.curr) {
+			switch (t->button.current) {
 			case BUTTON_EVENT_IN_AREA:
 				area |= AREA;
 				break;
@@ -1209,7 +1252,7 @@ bool
 tp_button_touch_active(const struct tp_dispatch *tp,
 		       const struct tp_touch *t)
 {
-	return t->button.state == BUTTON_STATE_AREA;
+	return t->button.state == BUTTON_STATE_AREA || t->button.has_moved;
 }
 
 bool
