@@ -45,7 +45,7 @@ enum evdev_event_type {
 	EVDEV_ABSOLUTE_TOUCH_DOWN = (1 << 0),
 	EVDEV_ABSOLUTE_MOTION = (1 << 1),
 	EVDEV_ABSOLUTE_TOUCH_UP = (1 << 2),
-	EVDEV_ABSOLUTE_MT= (1 << 3),
+	EVDEV_ABSOLUTE_MT = (1 << 3),
 	EVDEV_WHEEL = (1 << 4),
 	EVDEV_KEY = (1 << 5),
 	EVDEV_RELATIVE_MOTION = (1 << 6),
@@ -110,6 +110,7 @@ enum evdev_device_model {
 	EVDEV_MODEL_WACOM_TOUCHPAD = (1 << 7),
 	EVDEV_MODEL_ALPS_TOUCHPAD = (1 << 8),
 	EVDEV_MODEL_SYNAPTICS_SERIAL_TOUCHPAD = (1 << 9),
+	EVDEV_MODEL_TEST_DEVICE = (1 << 10),
 	EVDEV_MODEL_BOUNCING_KEYS = (1 << 11),
 	EVDEV_MODEL_LENOVO_X220_TOUCHPAD_FW81 = (1 << 12),
 	EVDEV_MODEL_LENOVO_CARBON_X1_6TH = (1 << 13),
@@ -157,21 +158,6 @@ enum evdev_debounce_state {
 	DEBOUNCE_ACTIVE,
 };
 
-enum mt_slot_state {
-	SLOT_STATE_NONE,
-	SLOT_STATE_BEGIN,
-	SLOT_STATE_UPDATE,
-	SLOT_STATE_END,
-};
-
-struct mt_slot {
-	bool dirty;
-	enum mt_slot_state state;
-	int32_t seat_slot;
-	struct device_coords point;
-	struct device_coords hysteresis_center;
-};
-
 struct evdev_device {
 	struct libinput_device base;
 
@@ -189,7 +175,8 @@ struct evdev_device {
 	bool is_mt;
 	bool is_suspended;
 	int dpi; /* HW resolution */
-	int trackpoint_range; /* trackpoint max delta */
+	double trackpoint_multiplier; /* trackpoint constant multiplier */
+	bool use_velocity_averaging; /* whether averaging should be applied on velocity calculation */
 	struct ratelimit syn_drop_limit; /* ratelimit for SYN_DROPPED logging */
 	struct ratelimit nonpointer_rel_limit; /* ratelimit for REL_* events from non-pointer devices */
 	uint32_t model_flags;
