@@ -198,7 +198,7 @@ START_TEST(device_disable_touchpad)
 	litest_assert_empty_queue(li);
 
 	litest_touch_down(dev, 0, 50, 50);
-	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10, 0);
+	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10);
 	litest_touch_up(dev, 0);
 
 	litest_assert_empty_queue(li);
@@ -230,7 +230,7 @@ START_TEST(device_disable_touch)
 	litest_assert_empty_queue(li);
 
 	litest_touch_down(dev, 0, 50, 50);
-	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10, 0);
+	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10);
 	litest_touch_up(dev, 0);
 
 	litest_assert_empty_queue(li);
@@ -254,7 +254,7 @@ START_TEST(device_disable_touch_during_touch)
 	device = dev->libinput_device;
 
 	litest_touch_down(dev, 0, 50, 50);
-	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10, 0);
+	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10);
 	litest_drain_events(li);
 
 	status = libinput_device_config_send_events_set_mode(device,
@@ -273,11 +273,11 @@ START_TEST(device_disable_touch_during_touch)
 
 	litest_assert_empty_queue(li);
 
-	litest_touch_move_to(dev, 0, 90, 90, 50, 50, 10, 0);
+	litest_touch_move_to(dev, 0, 90, 90, 50, 50, 10);
 	litest_touch_up(dev, 0);
 
 	litest_touch_down(dev, 0, 50, 50);
-	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10, 0);
+	litest_touch_move_to(dev, 0, 50, 50, 90, 90, 10);
 	litest_touch_up(dev, 0);
 
 	litest_assert_empty_queue(li);
@@ -1141,87 +1141,6 @@ START_TEST(device_accelerometer)
 }
 END_TEST
 
-START_TEST(device_udev_tag_alps)
-{
-	struct litest_device *dev = litest_current_device();
-	struct libinput_device *device = dev->libinput_device;
-	struct udev_device *d;
-	const char *prop;
-
-	d = libinput_device_get_udev_device(device);
-	prop = udev_device_get_property_value(d,
-					      "LIBINPUT_MODEL_ALPS_TOUCHPAD");
-
-	if (strstr(libinput_device_get_name(device), "ALPS"))
-		ck_assert_notnull(prop);
-	else
-		ck_assert(prop == NULL);
-
-	udev_device_unref(d);
-}
-END_TEST
-
-START_TEST(device_udev_tag_wacom)
-{
-	struct litest_device *dev = litest_current_device();
-	struct libinput_device *device = dev->libinput_device;
-	struct udev_device *d;
-	const char *prop;
-
-	d = libinput_device_get_udev_device(device);
-	prop = udev_device_get_property_value(d,
-					      "LIBINPUT_MODEL_WACOM_TOUCHPAD");
-
-	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_WACOM)
-		ck_assert_notnull(prop);
-	else
-		ck_assert(prop == NULL);
-
-	udev_device_unref(d);
-}
-END_TEST
-
-START_TEST(device_udev_tag_apple)
-{
-	struct litest_device *dev = litest_current_device();
-	struct libinput_device *device = dev->libinput_device;
-	struct udev_device *d;
-	const char *prop;
-
-	d = libinput_device_get_udev_device(device);
-	prop = udev_device_get_property_value(d,
-					      "LIBINPUT_MODEL_APPLE_TOUCHPAD");
-
-	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_APPLE)
-		ck_assert_notnull(prop);
-	else
-		ck_assert(prop == NULL);
-
-	udev_device_unref(d);
-}
-END_TEST
-
-START_TEST(device_udev_tag_synaptics_serial)
-{
-	struct litest_device *dev = litest_current_device();
-	struct libinput_device *device = dev->libinput_device;
-	struct udev_device *d;
-	const char *prop;
-
-	d = libinput_device_get_udev_device(device);
-	prop = udev_device_get_property_value(d,
-					      "LIBINPUT_MODEL_SYNAPTICS_SERIAL_TOUCHPAD");
-
-	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_SYNAPTICS_SERIAL &&
-	    libevdev_get_id_product(dev->evdev) == PRODUCT_ID_SYNAPTICS_SERIAL)
-		ck_assert_notnull(prop);
-	else
-		ck_assert(prop == NULL);
-
-	udev_device_unref(d);
-}
-END_TEST
-
 START_TEST(device_udev_tag_wacom_tablet)
 {
 	struct litest_device *dev = litest_current_device();
@@ -1458,7 +1377,7 @@ START_TEST(device_quirks_apple_magicmouse)
 
 	/* ensure we get no events from the touch */
 	litest_touch_down(dev, 0, 50, 50);
-	litest_touch_move_to(dev, 0, 50, 50, 80, 80, 10, 0);
+	litest_touch_move_to(dev, 0, 50, 50, 80, 80, 10);
 	litest_touch_up(dev, 0);
 	litest_assert_empty_queue(li);
 }
@@ -1627,10 +1546,6 @@ TEST_COLLECTION(device)
 	litest_add("device:wheel", device_wheel_only, LITEST_WHEEL, LITEST_RELATIVE|LITEST_ABSOLUTE|LITEST_TABLET);
 	litest_add_no_device("device:accelerometer", device_accelerometer);
 
-	litest_add("device:udev tags", device_udev_tag_alps, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:udev tags", device_udev_tag_wacom, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:udev tags", device_udev_tag_apple, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:udev tags", device_udev_tag_synaptics_serial, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("device:udev tags", device_udev_tag_wacom_tablet, LITEST_TABLET, LITEST_ANY);
 
 	litest_add_no_device("device:invalid rel events", device_nonpointer_rel);
