@@ -143,16 +143,15 @@ END_TEST
 static bool
 lid_switch_is_reliable(struct litest_device *dev)
 {
-	struct udev_device *udev_device;
-	const char *prop;
+	char *prop;
 	bool is_reliable = false;
 
-	udev_device = libinput_device_get_udev_device(dev->libinput_device);
-	prop = udev_device_get_property_value(udev_device,
-					      "LIBINPUT_ATTR_LID_SWITCH_RELIABILITY");
+	if (quirks_get_string(dev->quirks,
+			      QUIRK_ATTR_LID_SWITCH_RELIABILITY,
+			      &prop)) {
+		is_reliable = streq(prop, "reliable");
+	}
 
-	is_reliable = prop && streq(prop, "reliable");
-	udev_device_unref(udev_device);
 
 	return is_reliable;
 }
@@ -261,7 +260,7 @@ START_TEST(switch_disable_touchpad)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -270,7 +269,7 @@ START_TEST(switch_disable_touchpad)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
 
@@ -293,13 +292,13 @@ START_TEST(switch_disable_touchpad_during_touch)
 	litest_drain_events(li);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 5, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 5);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
 
 	litest_switch_action(sw, which, LIBINPUT_SWITCH_STATE_ON);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
-	litest_touch_move_to(touchpad, 0, 70, 50, 50, 50, 5, 1);
+	litest_touch_move_to(touchpad, 0, 70, 50, 50, 50, 5);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -331,11 +330,11 @@ START_TEST(switch_disable_touchpad_edge_scroll)
 	libinput_dispatch(li);
 	litest_assert_empty_queue(li);
 
-	litest_touch_move_to(touchpad, 0, 99, 20, 99, 80, 60, 10);
+	litest_touch_move_to(touchpad, 0, 99, 20, 99, 80, 60);
 	libinput_dispatch(li);
 	litest_assert_empty_queue(li);
 
-	litest_touch_move_to(touchpad, 0, 99, 80, 99, 20, 60, 10);
+	litest_touch_move_to(touchpad, 0, 99, 80, 99, 20, 60);
 	litest_touch_up(touchpad, 0);
 	libinput_dispatch(li);
 	litest_assert_empty_queue(li);
@@ -363,7 +362,7 @@ START_TEST(switch_disable_touchpad_edge_scroll_interrupt)
 	litest_touch_down(touchpad, 0, 99, 20);
 	libinput_dispatch(li);
 	litest_timeout_edgescroll();
-	litest_touch_move_to(touchpad, 0, 99, 20, 99, 30, 10, 10);
+	litest_touch_move_to(touchpad, 0, 99, 20, 99, 30, 10);
 	libinput_dispatch(li);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_AXIS);
 
@@ -401,7 +400,7 @@ START_TEST(switch_disable_touchpad_already_open)
 
 	/* default: switch is off - motion events */
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
 
@@ -410,7 +409,7 @@ START_TEST(switch_disable_touchpad_already_open)
 	litest_assert_empty_queue(li);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
 
@@ -439,7 +438,7 @@ START_TEST(switch_dont_resume_disabled_touchpad)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -448,7 +447,7 @@ START_TEST(switch_dont_resume_disabled_touchpad)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -474,7 +473,7 @@ START_TEST(switch_dont_resume_disabled_touchpad_external_mouse)
 	litest_drain_events(li);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -483,7 +482,7 @@ START_TEST(switch_dont_resume_disabled_touchpad_external_mouse)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -492,7 +491,7 @@ START_TEST(switch_dont_resume_disabled_touchpad_external_mouse)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -568,7 +567,7 @@ START_TEST(lid_open_on_key_touchpad_enabled)
 	litest_timeout_dwt_long();
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 70, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 70, 10);
 	litest_touch_up(touchpad, 0);
 	libinput_dispatch(li);
 
@@ -848,7 +847,7 @@ START_TEST(tablet_mode_disable_touchpad_on_init)
 	litest_drain_events(li);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_empty_queue(li);
 
@@ -858,7 +857,7 @@ START_TEST(tablet_mode_disable_touchpad_on_init)
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_SWITCH_TOGGLE);
 
 	litest_touch_down(touchpad, 0, 50, 50);
-	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10, 1);
+	litest_touch_move_to(touchpad, 0, 50, 50, 70, 50, 10);
 	litest_touch_up(touchpad, 0);
 	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
 
