@@ -40,9 +40,11 @@
 #include <signal.h>
 #include <stdbool.h>
 
+#include "libinput-versionsort.h"
 #include "libinput-util.h"
 #include "libinput-version.h"
 #include "libinput-git-version.h"
+#include "shared.h"
 
 static const int FILE_VERSION_NUMBER = 1;
 
@@ -213,8 +215,6 @@ print_evdev_event(struct record_context *ctx, struct input_event *ev)
 		unsigned long time, dt;
 
 		time = us2ms(tv2us(&ev->time));
-		if (last_ms == 0)
-			last_ms = time;
 		dt = time - last_ms;
 		last_ms = time;
 
@@ -716,6 +716,7 @@ buffer_gesture_event(struct record_context *ctx,
 static char *
 buffer_tablet_axes(struct libinput_event_tablet_tool *t)
 {
+	const int MAX_AXES = 10;
 	struct libinput_tablet_tool *tool;
 	char *s = NULL;
 	int idx = 0;
@@ -725,7 +726,7 @@ buffer_tablet_axes(struct libinput_event_tablet_tool *t)
 
 	tool = libinput_event_tablet_tool_get_tool(t);
 
-	strv = zalloc(10 * sizeof *strv);
+	strv = zalloc(MAX_AXES * sizeof *strv);
 
 	x = libinput_event_tablet_tool_get_x(t);
 	y = libinput_event_tablet_tool_get_y(t);
@@ -789,6 +790,8 @@ buffer_tablet_axes(struct libinput_event_tablet_tool *t)
 			goto out;
 	}
 
+	assert(idx < MAX_AXES);
+
 	s = strv_join(strv, ", ");
 out:
 	strv_free(strv);
@@ -1661,6 +1664,66 @@ out:
 	udev_unref(udev);
 }
 
+static void
+quirks_log_handler(struct libinput *this_is_null,
+		   enum libinput_log_priority priority,
+		   const char *format,
+		   va_list args)
+{
+}
+
+static void
+list_print(void *userdata, const char *val)
+{
+	struct record_context *ctx = userdata;
+
+	iprintf(ctx, "- %s\n", val);
+}
+
+static inline void
+print_device_quirks(struct record_context *ctx, struct record_device *dev)
+{
+	struct udev *udev = NULL;
+	struct udev_device *udev_device = NULL;
+	struct stat st;
+	struct quirks_context *quirks;
+	const char *data_path = LIBINPUT_QUIRKS_DIR;
+	const char *override_file = LIBINPUT_QUIRKS_OVERRIDE_FILE;
+
+	if (stat(dev->devnode, &st) < 0)
+		return;
+
+	quirks = quirks_init_subsystem(data_path,
+				       override_file,
+				       quirks_log_handler,
+				       NULL,
+				       QLOG_CUSTOM_LOG_PRIORITIES);
+	if (!quirks) {
+		fprintf(stderr,
+			"Failed to initialize the device quirks. "
+			"Please see the above errors "
+			"and/or re-run with --verbose for more details\n");
+		return;
+	}
+
+	udev = udev_new();
+	if (!udev)
+		goto out;
+
+	udev_device = udev_device_new_from_devnum(udev, 'c', st.st_rdev);
+	if (!udev_device)
+		goto out;
+
+	iprintf(ctx, "quirks:\n");
+	indent_push(ctx);
+
+	tools_list_device_quirks(quirks, udev_device, list_print, ctx);
+
+	indent_pop(ctx);
+out:
+	udev_device_unref(udev_device);
+	udev_unref(udev);
+}
 static inline void
 print_libinput_description(struct record_context *ctx,
 			   struct record_device *dev)
@@ -1716,6 +1779,7 @@ print_device_description(struct record_context *ctx, struct record_device *dev)
 
 	print_evdev_description(ctx, dev);
 	print_udev_properties(ctx, dev);
+	print_device_quirks(ctx, dev);
 	print_libinput_description(ctx, dev);
 }
 
