@@ -310,6 +310,13 @@ tools_open_device(const char *path, bool verbose, bool *grab)
 	return li;
 }
 
+static void
+tools_setenv_quirks_dir(void)
+{
+	if (tools_execdir_is_builddir(NULL, 0))
+		setenv("LIBINPUT_QUIRKS_DIR", LIBINPUT_QUIRKS_SRCDIR, 0);
+}
+
 struct libinput *
 tools_open_backend(enum tools_backend which,
 		   const char *seat_or_device,
@@ -318,6 +325,8 @@ tools_open_backend(enum tools_backend which,
 {
 	struct libinput *li;
 
+	tools_setenv_quirks_dir();
+
 	switch (which) {
 	case BACKEND_UDEV:
 		li = tools_open_udev(seat_or_device, verbose, grab);
@@ -466,16 +475,68 @@ out:
 	return is_touchpad;
 }
 
+/**
+ * Try to read the directory we're executing from and if it matches the
+ * builddir, return it.
+ *
+ * @param execdir_out If not NULL, set to the exec directory
+ * @param sz Size of execdir_out
+ *
+ * @return true if the execdir is the builddir, false otherwise.
+ *
+ * If execdir_out is NULL and szt is 0, it merely returns true/false.
+ */
+bool
+tools_execdir_is_builddir(char *execdir_out, size_t sz)
+{
+	char execdir[PATH_MAX] = {0};
+	char *pathsep;
+	ssize_t nread;
+
+	/* In the case of release builds, the builddir is
+	   the empty string */
+	if (streq(MESON_BUILD_ROOT, ""))
+		return false;
+
+	nread = readlink("/proc/self/exe", execdir, sizeof(execdir) - 1);
+	if (nread <= 0 || nread == sizeof(execdir) - 1)
+		return false;
+
+	/* readlink doesn't terminate the string and readlink says
+	   anything past sz is undefined */
+	execdir[++nread] = '\0';
+
+	pathsep = strrchr(execdir, '/');
+	if (!pathsep)
+		return false;
+
+	*pathsep = '\0';
+	if (!streq(execdir, MESON_BUILD_ROOT))
+		return false;
+
+	if (sz > 0) {
+		assert(execdir_out != NULL);
+		assert(sz >= (size_t)nread);
+		snprintf(execdir_out, nread, "%s", execdir);
+	}
+	return true;
+}
+
 static inline void
 setup_path(void)
 {
 	const char *path = getenv("PATH");
 	char new_path[PATH_MAX];
+	char builddir[PATH_MAX];
+	const char *extra_path = LIBINPUT_TOOL_PATH;
+
+	if (tools_execdir_is_builddir(builddir, sizeof(builddir)))
+		extra_path = builddir;
 
 	snprintf(new_path,
 		 sizeof(new_path),
 		 "%s:%s",
-		 LIBINPUT_TOOL_PATH,
+		 extra_path,
 		 path ? path : "");
 	setenv("PATH", new_path, 1);
 }
@@ -526,3 +587,152 @@ tools_exec_command(const char *prefix, int real_argc, char **real_argv)
 
 	return EXIT_FAILURE;
 }
+
+void
+tools_list_device_quirks(struct quirks_context *ctx,
+			 struct udev_device *device,
+			 void (*callback)(void *data, const char *str),
+			 void *userdata)
+{
+	char buf[256];
+
+	struct quirks *quirks;
+	enum quirk qlist[] = {
+		QUIRK_MODEL_ALPS_TOUCHPAD,
+		QUIRK_MODEL_APPLE_TOUCHPAD,
+		QUIRK_MODEL_APPLE_MAGICMOUSE,
+		QUIRK_MODEL_TABLET_NO_TILT,
+		QUIRK_MODEL_APPLE_TOUCHPAD_ONEBUTTON,
+		QUIRK_MODEL_TOUCHPAD_VISIBLE_MARKER,
+		QUIRK_MODEL_CYBORG_RAT,
+		QUIRK_MODEL_CHROMEBOOK,
+		QUIRK_MODEL_HP6910_TOUCHPAD,
+		QUIRK_MODEL_HP8510_TOUCHPAD,
+		QUIRK_MODEL_HP_PAVILION_DM4_TOUCHPAD,
+		QUIRK_MODEL_HP_STREAM11_TOUCHPAD,
+		QUIRK_MODEL_HP_ZBOOK_STUDIO_G3,
+		QUIRK_MODEL_TABLET_NO_PROXIMITY_OUT,
+		QUIRK_MODEL_LENOVO_SCROLLPOINT,
+		QUIRK_MODEL_LENOVO_X230,
+		QUIRK_MODEL_LENOVO_T450_TOUCHPAD,
+		QUIRK_MODEL_TABLET_MODE_NO_SUSPEND,
+		QUIRK_MODEL_LENOVO_CARBON_X1_6TH,
+		QUIRK_MODEL_TRACKBALL,
+		QUIRK_MODEL_LOGITECH_MARBLE_MOUSE,
+		QUIRK_MODEL_BOUNCING_KEYS,
+		QUIRK_MODEL_SYNAPTICS_SERIAL_TOUCHPAD,
+		QUIRK_MODEL_SYSTEM76_BONOBO,
+		QUIRK_MODEL_CLEVO_W740SU,
+		QUIRK_MODEL_SYSTEM76_GALAGO,
+		QUIRK_MODEL_SYSTEM76_KUDU,
+		QUIRK_MODEL_WACOM_TOUCHPAD,
+
+
+		QUIRK_ATTR_SIZE_HINT,
+		QUIRK_ATTR_TOUCH_SIZE_RANGE,
+		QUIRK_ATTR_PALM_SIZE_THRESHOLD,
+		QUIRK_ATTR_LID_SWITCH_RELIABILITY,
+		QUIRK_ATTR_KEYBOARD_INTEGRATION,
+		QUIRK_ATTR_TPKBCOMBO_LAYOUT,
+		QUIRK_ATTR_PRESSURE_RANGE,
+		QUIRK_ATTR_PALM_PRESSURE_THRESHOLD,
+		QUIRK_ATTR_RESOLUTION_HINT,
+		QUIRK_ATTR_TRACKPOINT_MULTIPLIER,
+		QUIRK_ATTR_THUMB_PRESSURE_THRESHOLD,
+		QUIRK_ATTR_USE_VELOCITY_AVERAGING,
+		QUIRK_ATTR_THUMB_SIZE_THRESHOLD,
+		QUIRK_ATTR_MSC_TIMESTAMP,
+	};
+	enum quirk *q;
+
+	quirks = quirks_fetch_for_device(ctx, device);
+	if (!quirks)
+		return;
+
+	ARRAY_FOR_EACH(qlist, q) {
+		const char *name;
+		struct quirk_dimensions dim;
+		struct quirk_range r;
+		uint32_t v;
+		char *s;
+		double d;
+
+		if (!quirks_has_quirk(quirks, *q))
+			continue;
+
+		name = quirk_get_name(*q);
+
+		switch (*q) {
+		case QUIRK_MODEL_ALPS_TOUCHPAD:
+		case QUIRK_MODEL_APPLE_TOUCHPAD:
+		case QUIRK_MODEL_APPLE_MAGICMOUSE:
+		case QUIRK_MODEL_TABLET_NO_TILT:
+		case QUIRK_MODEL_APPLE_TOUCHPAD_ONEBUTTON:
+		case QUIRK_MODEL_TOUCHPAD_VISIBLE_MARKER:
+		case QUIRK_MODEL_CYBORG_RAT:
+		case QUIRK_MODEL_CHROMEBOOK:
+		case QUIRK_MODEL_HP6910_TOUCHPAD:
+		case QUIRK_MODEL_HP8510_TOUCHPAD:
+		case QUIRK_MODEL_HP_PAVILION_DM4_TOUCHPAD:
+		case QUIRK_MODEL_HP_STREAM11_TOUCHPAD:
+		case QUIRK_MODEL_HP_ZBOOK_STUDIO_G3:
+		case QUIRK_MODEL_TABLET_NO_PROXIMITY_OUT:
+		case QUIRK_MODEL_LENOVO_SCROLLPOINT:
+		case QUIRK_MODEL_LENOVO_X230:
+		case QUIRK_MODEL_LENOVO_T450_TOUCHPAD:
+		case QUIRK_MODEL_TABLET_MODE_NO_SUSPEND:
+		case QUIRK_MODEL_LENOVO_CARBON_X1_6TH:
+		case QUIRK_MODEL_TRACKBALL:
+		case QUIRK_MODEL_LOGITECH_MARBLE_MOUSE:
+		case QUIRK_MODEL_BOUNCING_KEYS:
+		case QUIRK_MODEL_SYNAPTICS_SERIAL_TOUCHPAD:
+		case QUIRK_MODEL_SYSTEM76_BONOBO:
+		case QUIRK_MODEL_CLEVO_W740SU:
+		case QUIRK_MODEL_SYSTEM76_GALAGO:
+		case QUIRK_MODEL_SYSTEM76_KUDU:
+		case QUIRK_MODEL_WACOM_TOUCHPAD:
+			snprintf(buf, sizeof(buf), "%s=1", name);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_SIZE_HINT:
+		case QUIRK_ATTR_RESOLUTION_HINT:
+			quirks_get_dimensions(quirks, *q, &dim);
+			snprintf(buf, sizeof(buf), "%s=%ldx%ld", name, dim.x, dim.y);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_TOUCH_SIZE_RANGE:
+		case QUIRK_ATTR_PRESSURE_RANGE:
+			quirks_get_range(quirks, *q, &r);
+			snprintf(buf, sizeof(buf), "%s=%d:%d", name, r.upper, r.lower);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_PALM_SIZE_THRESHOLD:
+		case QUIRK_ATTR_PALM_PRESSURE_THRESHOLD:
+		case QUIRK_ATTR_THUMB_PRESSURE_THRESHOLD:
+		case QUIRK_ATTR_THUMB_SIZE_THRESHOLD:
+			quirks_get_uint32(quirks, *q, &v);
+			snprintf(buf, sizeof(buf), "%s=%u", name, v);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_LID_SWITCH_RELIABILITY:
+		case QUIRK_ATTR_KEYBOARD_INTEGRATION:
+		case QUIRK_ATTR_TPKBCOMBO_LAYOUT:
+		case QUIRK_ATTR_MSC_TIMESTAMP:
+			quirks_get_string(quirks, *q, &s);
+			snprintf(buf, sizeof(buf), "%s=%s", name, s);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_TRACKPOINT_MULTIPLIER:
+			quirks_get_double(quirks, *q, &d);
+			snprintf(buf, sizeof(buf), "%s=%0.2f\n", name, d);
+			callback(userdata, buf);
+			break;
+		case QUIRK_ATTR_USE_VELOCITY_AVERAGING:
+			snprintf(buf, sizeof(buf), "%s=1", name);
+			callback(userdata, buf);
+			break;
+		}
+	}
+
+	quirks_unref(quirks);
+}
